@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 //TESTED_COMPONENT=src/location
 
@@ -25,6 +25,8 @@
 #include "logfilepositionsource.h"
 #include "positionconsumerthread.h"
 
+#include <algorithm>
+
 QT_USE_NAMESPACE
 #define UPDATE_INTERVAL 50
 
@@ -159,10 +161,10 @@ private slots:
         QVERIFY(obj != nullptr);
         QCOMPARE(obj->sourceName(), QStringLiteral("positionpoll"));
 
-        QList<QGeoAreaMonitorInfo> list = obj->activeMonitors();
+        const QList<QGeoAreaMonitorInfo> list = obj->activeMonitors();
         if (list.size() > 0) {
             //cleanup installed monitors
-            foreach (const QGeoAreaMonitorInfo& info, list) {
+            for (const QGeoAreaMonitorInfo& info : list) {
                 QVERIFY(obj->stopMonitoring(info));
             }
         }
@@ -455,32 +457,19 @@ private slots:
         mon3.setArea(r_above);
         QVERIFY(obj->startMonitoring(mon3));
 
-        QList<QGeoAreaMonitorInfo> results = obj->activeMonitors();
-        QCOMPARE(results.size(), 3);
-        foreach (const QGeoAreaMonitorInfo& info, results) {
-            QVERIFY(info == mon || info == mon2 || info == mon3);
-        }
+#define CHECK(o, args, ...) do { \
+            const QList<QGeoAreaMonitorInfo> results = o ->activeMonitors args; \
+            const std::initializer_list<QGeoAreaMonitorInfo> expected = __VA_ARGS__ ; \
+            QCOMPARE(results.size(), int(expected.size())); \
+            QVERIFY(std::is_permutation(results.begin(), results.end(), \
+                                        expected.begin(), expected.end())); \
+        } while (false)
 
-        results = obj->activeMonitors(QGeoShape());
-        QCOMPARE(results.size(), 0);
-
-        results = obj->activeMonitors(QGeoRectangle(QGeoCoordinate(1,1),0.2, 0.2));
-        QCOMPARE(results.size(), 2);
-        foreach (const QGeoAreaMonitorInfo& info, results) {
-            QVERIFY(info == mon || info == mon2);
-        }
-
-        results = obj->activeMonitors(QGeoCircle(QGeoCoordinate(1,1),1000));
-        QCOMPARE(results.size(), 2);
-        foreach (const QGeoAreaMonitorInfo& info, results) {
-            QVERIFY(info == mon || info == mon2);
-        }
-
-        results = obj->activeMonitors(QGeoCircle(QGeoCoordinate(2,1),1000));
-        QCOMPARE(results.size(), 1);
-        foreach (const QGeoAreaMonitorInfo& info, results) {
-            QVERIFY(info == mon3);
-        }
+        CHECK(obj, (), {mon, mon2, mon3});
+        CHECK(obj, (QGeoShape{}), {});
+        CHECK(obj, (QGeoRectangle{QGeoCoordinate{1, 1}, 0.2, 0.2}), {mon, mon2});
+        CHECK(obj, (QGeoCircle{QGeoCoordinate{1, 1}, 1000}), {mon, mon2});
+        CHECK(obj, (QGeoCircle{QGeoCoordinate{2, 1},1000}), {mon3});
 
         //same as above except that we use a different monitor source object instance
         //all monitor objects of same type share same active monitors
@@ -489,32 +478,13 @@ private slots:
         QVERIFY(secondObj != nullptr);
         QCOMPARE(secondObj->sourceName(), QStringLiteral("positionpoll"));
 
-        results = secondObj->activeMonitors();
-        QCOMPARE(results.size(), 3);
-        foreach (const QGeoAreaMonitorInfo& info, results) {
-            QVERIFY(info == mon || info == mon2 || info == mon3);
-        }
+        CHECK(secondObj, (), {mon, mon2, mon3});
+        CHECK(secondObj, (QGeoShape{}), {});
+        CHECK(secondObj, (QGeoRectangle{QGeoCoordinate{1, 1}, 0.2, 0.2}), {mon, mon2});
+        CHECK(secondObj, (QGeoCircle{QGeoCoordinate{1, 1}, 1000}), {mon, mon2});
+        CHECK(secondObj, (QGeoCircle{QGeoCoordinate{2, 1}, 1000}), {mon3});
 
-        results = secondObj->activeMonitors(QGeoShape());
-        QCOMPARE(results.size(), 0);
-
-        results = secondObj->activeMonitors(QGeoRectangle(QGeoCoordinate(1,1),0.2, 0.2));
-        QCOMPARE(results.size(), 2);
-        foreach (const QGeoAreaMonitorInfo& info, results) {
-            QVERIFY(info == mon || info == mon2);
-        }
-
-        results = secondObj->activeMonitors(QGeoCircle(QGeoCoordinate(1,1),1000));
-        QCOMPARE(results.size(), 2);
-        foreach (const QGeoAreaMonitorInfo& info, results) {
-            QVERIFY(info == mon || info == mon2);
-        }
-
-        results = secondObj->activeMonitors(QGeoCircle(QGeoCoordinate(2,1),1000));
-        QCOMPARE(results.size(), 1);
-        foreach (const QGeoAreaMonitorInfo& info, results) {
-            QVERIFY(info == mon3);
-        }
+#undef CHECK
     }
 
     void tst_testExpiryTimeout()
@@ -590,8 +560,8 @@ private slots:
         QCOMPARE(obj->sourceName(), QStringLiteral("positionpoll"));
         obj->setObjectName("firstObject");
         QSignalSpy enteredSpy(obj.get(),
-                              SIGNAL(areaEntered(QGeoAreaMonitorInfo, QGeoPositionInfo)));
-        QSignalSpy exitedSpy(obj.get(), SIGNAL(areaExited(QGeoAreaMonitorInfo, QGeoPositionInfo)));
+                              SIGNAL(areaEntered(QGeoAreaMonitorInfo,QGeoPositionInfo)));
+        QSignalSpy exitedSpy(obj.get(), SIGNAL(areaExited(QGeoAreaMonitorInfo,QGeoPositionInfo)));
 
         // using this -> no need for smart pointer
         LogFilePositionSource *source = new LogFilePositionSource(m_fileData, this);
@@ -604,9 +574,9 @@ private slots:
         QVERIFY(secondObj != nullptr);
         QCOMPARE(secondObj->sourceName(), QStringLiteral("positionpoll"));
         QSignalSpy enteredSpy2(secondObj.get(),
-                               SIGNAL(areaEntered(QGeoAreaMonitorInfo, QGeoPositionInfo)));
+                               SIGNAL(areaEntered(QGeoAreaMonitorInfo,QGeoPositionInfo)));
         QSignalSpy exitedSpy2(secondObj.get(),
-                              SIGNAL(areaExited(QGeoAreaMonitorInfo, QGeoPositionInfo)));
+                              SIGNAL(areaExited(QGeoAreaMonitorInfo,QGeoPositionInfo)));
         secondObj->setObjectName("secondObject");
 
         QGeoAreaMonitorInfo infoRectangle("Rectangle");
@@ -726,8 +696,8 @@ private slots:
         QCOMPARE(obj->sourceName(), QStringLiteral("positionpoll"));
         obj->setObjectName("firstObject");
         QSignalSpy enteredSpy(obj.get(),
-                              SIGNAL(areaEntered(QGeoAreaMonitorInfo, QGeoPositionInfo)));
-        QSignalSpy exitedSpy(obj.get(), SIGNAL(areaExited(QGeoAreaMonitorInfo, QGeoPositionInfo)));
+                              SIGNAL(areaEntered(QGeoAreaMonitorInfo,QGeoPositionInfo)));
+        QSignalSpy exitedSpy(obj.get(), SIGNAL(areaExited(QGeoAreaMonitorInfo,QGeoPositionInfo)));
 
         std::unique_ptr<QGeoAreaMonitorSource> obj2(
                 QGeoAreaMonitorSource::createSource(QStringLiteral("positionpoll"), 0));
@@ -735,9 +705,9 @@ private slots:
         QCOMPARE(obj2->sourceName(), QStringLiteral("positionpoll"));
         obj2->setObjectName("secondObject");
         QSignalSpy enteredSpy2(obj2.get(),
-                               SIGNAL(areaEntered(QGeoAreaMonitorInfo, QGeoPositionInfo)));
+                               SIGNAL(areaEntered(QGeoAreaMonitorInfo,QGeoPositionInfo)));
         QSignalSpy exitedSpy2(obj2.get(),
-                              SIGNAL(areaExited(QGeoAreaMonitorInfo, QGeoPositionInfo)));
+                              SIGNAL(areaExited(QGeoAreaMonitorInfo,QGeoPositionInfo)));
 
         // using this -> no need for smart pointer
         LogFilePositionSource *source = new LogFilePositionSource(m_fileData, this);
@@ -859,6 +829,9 @@ private slots:
 
     void multipleThreads()
     {
+#if defined(Q_OS_MACOS) && defined(Q_PROCESSOR_ARM)
+        QSKIP("This test randomly hangs on macOS ARM (QTBUG-110931)");
+#endif
         std::unique_ptr<QGeoAreaMonitorSource> obj(
                 QGeoAreaMonitorSource::createSource(QStringLiteral("positionpoll"), 0));
         QVERIFY(obj != nullptr);
@@ -899,13 +872,13 @@ private slots:
         singleShot_enter.setArea(QGeoRectangle(QGeoCoordinate(-27.67, 153.093), 0.2, 0.2));
         QVERIFY(singleShot_enter.isValid());
         QVERIFY(obj->requestUpdate(singleShot_enter,
-                                   SIGNAL(areaEntered(QGeoAreaMonitorInfo, QGeoPositionInfo))));
+                                   SIGNAL(areaEntered(QGeoAreaMonitorInfo,QGeoPositionInfo))));
 
         QGeoAreaMonitorInfo singleShot_exit("SingleShot_on_Exited");
         singleShot_exit.setArea(QGeoRectangle(QGeoCoordinate(-27.70, 153.093), 0.2, 0.2));
         QVERIFY(singleShot_exit.isValid());
         QVERIFY(obj->requestUpdate(singleShot_exit,
-                                   SIGNAL(areaExited(QGeoAreaMonitorInfo, QGeoPositionInfo))));
+                                   SIGNAL(areaExited(QGeoAreaMonitorInfo,QGeoPositionInfo))));
 
         // wait until we read all data
         QTRY_COMPARE_WITH_TIMEOUT(noDataSpy.size(), 1, 5000);
@@ -945,6 +918,26 @@ private slots:
         QCOMPARE(obj->backendProperty(DummyMonitorSource::kTestProperty), 10);
         QCOMPARE(obj->backendProperty(invalidProperty), QVariant());
     }
+
+    void randomSignalName()
+    {
+        auto src = std::unique_ptr<QGeoAreaMonitorSource>(
+                QGeoAreaMonitorSource::createSource(QStringLiteral("positionpoll"), nullptr));
+        QVERIFY(src);
+
+        QGeoAreaMonitorInfo infoCircle("Circle");
+        infoCircle.setArea(QGeoCircle(QGeoCoordinate(-27.70, 153.093), 10000));
+        QVERIFY(infoCircle.isValid());
+
+        // empty signal -> invalid
+        QVERIFY(!src->requestUpdate(infoCircle, ""));
+
+        // non-existent single-char signal -> invalid
+        QVERIFY(!src->requestUpdate(infoCircle, "a"));
+
+        // non-existent multi-char signal -> invalid
+        QVERIFY(!src->requestUpdate(infoCircle, "abcd"));
+    }
 };
 
 
