@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 //TESTED_COMPONENT=src/location
 
@@ -2243,6 +2218,11 @@ void tst_QGeoRectangle::unite_data()
             << QGeoRectangle(QGeoCoordinate(30, 160), QGeoCoordinate(-30, 170))
             << QGeoRectangle(QGeoCoordinate(30, 160), QGeoCoordinate(-30, -170))
             << QGeoRectangle(QGeoCoordinate(30, 160), QGeoCoordinate(-30, -170));
+
+    QTest::newRow("wrappinng, one containing other, overlapping center")
+            << QGeoRectangle(QGeoCoordinate(30, 170), QGeoCoordinate(-30, -30))
+            << QGeoRectangle(QGeoCoordinate(30, -140), QGeoCoordinate(-30, -50))
+            << QGeoRectangle(QGeoCoordinate(30, 170), QGeoCoordinate(-30, -30));
 }
 
 
