@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QMetaType>
 #include <QSignalSpy>
@@ -734,8 +734,12 @@ void TestQGeoSatelliteInfoSource::bindings()
 {
     CHECK_SOURCE_VALID;
 
-    QTestPrivate::testReadWritePropertyBasics<QGeoSatelliteInfoSource, int>(*m_source, 1000, 2000,
-                                                                            "updateInterval");
+    QTestPrivate::testReadWritePropertyBasics<QGeoSatelliteInfoSource, int>(
+            *m_source, 1000, 2000, "updateInterval",
+            []() {
+                return std::unique_ptr<QGeoSatelliteInfoSource>(
+                        QGeoSatelliteInfoSource::createDefaultSource(nullptr));
+            });
 }
 
 #include "testqgeosatelliteinfosource.moc"
